use crate::article_view::ArticleTheme;
use glib::subclass::*;
use gtk4::{CompositeTemplate, ListBoxRow, Popover, Widget, prelude::*, subclass::prelude::*};
use once_cell::sync::Lazy;

mod imp {
    use super::*;

    #[derive(Debug, Default, CompositeTemplate)]
    #[template(file = "data/resources/ui_templates/settings/theme_chooser.blp")]
    pub struct ThemeChooser {}

    #[glib::object_subclass]
    impl ObjectSubclass for ThemeChooser {
        const NAME: &'static str = "ThemeChooser";
        type ParentType = Popover;
        type Type = super::ThemeChooser;

        fn class_init(klass: &mut Self::Class) {
            klass.bind_template();
            klass.bind_template_callbacks();
        }

        fn instance_init(obj: &InitializingObject<Self>) {
            obj.init_template();
        }
    }

    impl ObjectImpl for ThemeChooser {
        fn signals() -> &'static [Signal] {
            static SIGNALS: Lazy<Vec<Signal>> = Lazy::new(|| {
                vec![
                    Signal::builder("article-theme-changed")
                        .param_types([ArticleTheme::static_type()])
                        .build(),
                ]
            });
            SIGNALS.as_ref()
        }
    }

    impl WidgetImpl for ThemeChooser {}

    impl PopoverImpl for ThemeChooser {}

    #[gtk4::template_callbacks]
    impl ThemeChooser {
        #[template_callback]
        fn on_row_activated(&self, row: &ListBoxRow) {
            let obj = self.obj();
            let theme = ArticleTheme::from_str(row.widget_name().as_str());

            obj.popdown();
            obj.emit_by_name::<()>("article-theme-changed", &[&theme]);
        }
    }
}

glib::wrapper! {
    pub struct ThemeChooser(ObjectSubclass<imp::ThemeChooser>)
        @extends Widget, Popover;
}

impl Default for ThemeChooser {
    fn default() -> Self {
        glib::Object::new::<Self>()
    }
}
